// Demo: Dual CAN-BUS Shields, Data Pass-through
// Written by: Cory J. Fowler
// January 31st 2014
// This examples the ability of this library to support more than one MCP2515 
// based CAN interface.
// https://github.com/coryjfowler/MCP_CAN_lib/blob/master/examples/Dual_CAN/Dual_CAN.ino
//
// Modified by Copperhill Technologies (2023) for the use with 
// the espBerry and RPi CAN HATs.
//
#include <Arduino.h>
#include "mcp2515.h"

// CAN Ports Functions
// -----------------------------------------------------------------------------
#define DUAL_CAN        1   // 0 - Single CAN operation; 1 - Dual CAN operation

extern byte canInit(const int canPort, const CAN_SPEED canSpeed, 
                    const int canOpMode);
extern byte canRx(byte cPort, unsigned long* lMsgID, byte* cMessageIDFormat, 
                  byte* cData, byte* cDataLen);
extern byte canTx(byte cPort, unsigned long lMsgID, byte cMessageIDFormat, 
                  byte* cData, byte cDataLen);
// -----------------------------------------------------------------------------

int nCounter = 0;

byte txBuf0[] = {0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55};
unsigned long lMsgID0 = 0x18FF0992;

#if DUAL_CAN == 1
byte txBuf1[] = {0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA};
unsigned long lMsgID1 = 0x18FF0993;
#endif

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------
void setup()
{
    // Initialize Serial Monitor
    Serial.begin(115200);

    // Initialize the CAN ports
    if(canInit(0, CAN_250KBPS, CAN_OP_NORMAL) == CAN_OK)
        Serial.println("CAN0 Initialized.");
    else
        Serial.println("CAN0 Initialization Error.");
#if DUAL_CAN == 1
    if(canInit(1, CAN_250KBPS, CAN_OP_NORMAL) == CAN_OK)
        Serial.println("CAN1 Initialized.");
    else
        Serial.println("CAN1 Initialization Error.");
#endif

}// end setup

// -----------------------------------------------------------------------------
// Loop
// -----------------------------------------------------------------------------
void loop()
{
    // Declarations
    unsigned long lMsgID;
    byte bExtFormat;
    byte cData[8];
    byte cDataLen;

    // For demo purposes...
    delay(10);
    if(++nCounter == 100)
    {
        nCounter = 0;
        
        if(canTx(0, lMsgID0, CAN_ID_EXT, txBuf0, 8) != CAN_OK)
            Serial.println("CAN0: Transmit Error.");
#if DUAL_CAN == 1        
        if(canTx(1, lMsgID1, CAN_ID_EXT, txBuf1, 8) != CAN_OK)
            Serial.println("CAN1: Transmit Error.");
#endif
    }// end if

    // Check for data at CAN port 0
    if(canRx(0, &lMsgID, &bExtFormat, cData, &cDataLen) == CAN_OK)
    {
        Serial.print("CAN0:");
        Serial.print(lMsgID, HEX);
        Serial.print(" ID:");
        Serial.print(bExtFormat);
        Serial.print(" Data:");

        for(int nIndex = 0; nIndex < cDataLen; nIndex++)
        {
          Serial.print(cData[nIndex], HEX);
          Serial.print(" ");

        }// end if

        Serial.println("");

    }// end if

#if DUAL_CAN == 1
    // Check for data at CAN port 1
    if(canRx(1, &lMsgID, &bExtFormat, cData, &cDataLen) == CAN_OK)
    {
        Serial.print("CAN1:");
        Serial.print(lMsgID, HEX);
        Serial.print(" ID:");
        Serial.print(bExtFormat);
        Serial.print(" Data:");

        for(int nIndex = 0; nIndex < cDataLen; nIndex++)
        {
          Serial.print(cData[nIndex], HEX);
          Serial.print(" ");

        }// end for

        Serial.println("");

    }// end if
#endif

}// end loop

// -----------------------------------------------------------------------------
// END FILE
// -----------------------------------------------------------------------------
